<?php
/*
	Plugin Name: Fidgetr
	Plugin URI: http://langui.sh/fidgetr
	Description: A themeable Flickr widget (multi-widget supported) that can fetch pictures and comments and style them in a myriad of attractive ways.  Catch new releases and other information about my plugins by following <a href="http://twitter.com/reaperhulk" target="_blank">@reaperhulk</a> on Twitter.
	Version: 2.5.3
	Author: Paul Kehrer
	Author URI: http://langui.sh/
	Text Domain: fidgetr
*/
if(version_compare(PHP_VERSION, '5.0.0','<')) {
	die(sprintf(__('You are currently running PHP version %s and you must have at least PHP 5.0.x in order to use Fidgetr!', 'fidgetr'), PHP_VERSION));
}

require_once("flickrApi.class.php");

define('FIDGETR_VERSION','2.5.3');

if (!defined('WP_CONTENT_URL')) {
	define('WP_CONTENT_URL', get_option('siteurl') . '/wp-content');
}

class fidgetr_widget extends WP_Widget {
	private $theme_choices = array();
	private $plugin_domain = 'fidgetr';
	private $fidgetr_content_url = '';

	//constructor
	function fidgetr_widget() {
		$widget_opts = array('classname' => 'widget_fidgetr', 'description' => __( 'A simple and beautiful Flickr widget.', 'fidgetr'),'id_base' => 'fidgetr-widget' );
		$this->WP_Widget('fidgetr',__('Fidgetr', 'fidgetr'), $widget_opts);

		add_action('wp_print_styles', array($this, 'include_css')); //add css to head
		add_action('init', array($this, 'include_js')); //add js to head
		add_action('admin_init', array($this,'fidgetr_admin_init'));

		add_action('wp_ajax_fidgetr_user_check_ajax', array($this, 'user_check_ajax'));
		add_action('wp_ajax_fidgetr_photoset_fetch_ajax', array($this, 'photoset_fetch_ajax'));

		//load list of available themes.  the whole widget will fail if the themes dir is missing.
		$this->theme_choices = scandir(dirname(__FILE__)."/themes/");
		foreach($this->theme_choices as $key=>$value) {
			if(strlen($value) > 10 && strpos($value,".theme.php") > 0) {
				$this->theme_choices[$key] = substr($value,0,strpos($value,'.theme.php'));
			} else {
				unset($this->theme_choices[$key]);
			}
		}

		//load i18n strings
		load_plugin_textdomain($this->plugin_domain, PLUGINDIR.'/'.dirname(plugin_basename(__FILE__)).'/lang');
		
		//define a new WP_CONTENT_URL that handles HTTPS
		$this->fidgetr_content_url = (empty($_SERVER['HTTPS'])) ? WP_CONTENT_URL.'/plugins/fidgetr' : str_replace("http://", "https://", WP_CONTENT_URL.'/plugins/fidgetr');
	}

	public function include_css() {
		$all_instances = $this->get_settings();

		foreach($all_instances as $instance) {
			//for global slimbox.
			if($instance['slimbox'] == 1) {
				$slimboxpath = $this->fidgetr_content_url . '/themes/theme_includes/slimbox2/';
				wp_enqueue_style('slimbox2', $slimboxpath.'slimbox2.css');
			}

			//build the URL path for includes
			$path = $this->fidgetr_content_url . '/themes/theme_includes/'.$instance['theme'] . '/';

			if(!file_exists(dirname(__FILE__)."/themes/theme_includes/{$instance['theme']}")) {
				return; //nothing to include
			}
			$includes = scandir(dirname(__FILE__) . "/themes/theme_includes/{$instance['theme']}/");
			foreach($includes as $value) {
				if(strpos($value,".css") > 0) {
					$handle = substr($value,0,strpos($value,'.css'));
					wp_enqueue_style($handle, $path.$value);
				}
			}
		}
	}

	public function include_js() {
		$all_instances = $this->get_settings();

		foreach($all_instances as $instance) {
			//for global slimbox.
			if($instance['slimbox'] == 1) {
				$slimboxpath = $this->fidgetr_content_url .'/themes/theme_includes/slimbox2/';
				wp_enqueue_script( 'slimbox2', $slimboxpath.'slimbox-2.03-jquery.js', 'jquery', '2.03');
			}

			//build the URL path for includes
			$path = $this->fidgetr_content_url .'/themes/theme_includes/'.$instance['theme'].'/';

			if (!file_exists(dirname(__FILE__)."/themes/theme_includes/{$instance['theme']}")) {
				return; //nothing to include
			}
			$includes = scandir(dirname(__FILE__)."/themes/theme_includes/{$instance['theme']}/");
			foreach($includes as $value) {
				if(strpos($value,".js") > 0) {
					/*
					in fidgetr javascript files have a very strict naming structure to
					allow proper queueing via WP and reduce conflicts with other plugins.
					your file must be named handle-ver-deps.js  do not use - or _ in the names 
					as they are delimiters.  additionally, ver must be numeric.  multiple deps can 
					be provided via underscore (ex: handle-ver-dep1_dep2_dep3.js)
					*/
					preg_match("/^(.+?)-([^\-]+)(?:-(.+))?\.js$/",$value,$matches);
					if(isset($matches[3]) && strpos($matches[3],"_") == 0) {
						$deps = array($matches[3]);
					} elseif(strpos($matches[3],"_") > 0) {
						$deps = explode("_",$matches[3]);
					} else {
						$deps = false;
					}
					(is_numeric($matches[2]))?$ver=$matches[2]:$ver=false;
					// enqueue script takes the following args
					// handle, path, dependencies, version
					wp_enqueue_script( $matches[1], $path.$value, $deps, $ver);
				}
			}
		}
	}

	public function fidgetr_admin_init() {
		$admin_js_url = $this->fidgetr_content_url . '/fidgetr_ajax.js';
		wp_enqueue_script( 'fidgetr_ajax', $admin_js_url, array('jquery'));
	}

	public function user_check_ajax() {
		$flickr = new flickrApi;
		if($flickr->getNsid($_POST['username'])) {
			echo '<span style="color:green">'.__('Username is valid.', 'fidgetr').'</span>';
		} else {
			echo '<span style="color:red">'.__('Username or email is invalid. Please check your Flickr profile page to confirm your information.','fidgetr').'</span>';
		}
	}

	public function photoset_fetch_ajax() {
		$f = new flickrApi;
		$f->getNsid($_POST['username']);
		$f->getPhotosets();
		$photosets[0] = __('Latest Photos','fidgetr');
		if(is_array($f->photosetArr)) {
			foreach($f->photosetArr as $value) {
				$photosets[$value['id']] = $value['title']['_content'];
			}
		}
		foreach($photosets as $key=>$value) {
			if($key == $instance['photoset']) {
				$selected = 'selected="selected"';
			}
			echo "<option value=\"$key\" $selected>$value</option>\n";
			unset($selected);
		}
	}

	function widget($args, $instance) {
		// outputs the content of the widget
		// $args is an array of strings that help widgets to conform to
		// the active theme: before_widget, before_title, after_widget,
		// and after_title are the array keys. Default tags: li and h2.
		extract($args);

		// actual output of widget
		echo $before_widget;
		$title = empty($instance['title']) ? '&nbsp;' : apply_filters('widget_title', $instance['title']);
		if ( !empty( $title ) ) { echo $before_title . $title . $after_title; };
		$this->fidgetr_images($instance);
		echo $after_widget;
	}

	function fidgetr_images($instance) {
		//timing functions
		$start = microtime(true);

		if(isset($instance['datacache'])) {
			$dataCache = unserialize($instance['datacache']);
		}
		if( (time() - $dataCache['lastupdate'] > $instance['cachetime']) ||
		(!isset($dataCache)) ||
		(!isset($dataCache['urlArr'])) ||
		($dataCache['username'] != $instance['username']) ||
		($dataCache['num'] != $instance['num']) ||
		($dataCache['commentsflag'] != $instance['commentsflag']) ||
		($dataCache['theme'] != $instance['theme']) ||
		($dataCache['photoset'] != $instance['photoset']) ||
		($dataCache['showtitle'] != $instance['showtitle']) ||
		($dataCache['randomize'] != $instance['randomize'])
		) {
			//if it has been more than cachetime, there is no data cache, certain options have changed (# of photos, username, comment flag, theme), or the urlArr is empty we need to fetch data.
			$f = new flickrApi();
			$f->getNsid($instance['username']);
			$f->randomize = $instance['randomize'];
			$f->num = (int)$instance['num'];
			if($instance['photoset'] == 0) {
				$f->getPublicPhotos();
			} else {
				$f->getPhotosFromPhotoset($instance['photoset']);
			}
			if($instance['commentsflag']) {
				$f->getPhotoComments();
			}
			$instance['generated'] = date('m/d/Y G:i:s',time());
			//write a cache file to prevent constant queries to flickr.  no conflict with wp super cache.
			$instance['datacache'] = serialize(
				array(
					'lastupdate'=>time(),
					'num'=>$instance['num'],
					'username'=>$instance['username'],
					'commentsflag'=>$instance['commentsflag'],
					'urlArr'=>$f->urlArr,
					'theme'=>$instance['theme'],
					'photoset'=>$instance['photoset'],
					'showtitle'=>$instance['showtitle'],
					'randomize'=>$instance['randomize']
				)
			);
			$this->save_instance_settings($instance);
		} else {
			$f = new flickrApi();
			$f->urlArr = $dataCache['urlArr'];
		}
		$this->renderJsonOutput($f->urlArr);
		require(dirname(__FILE__)."/themes/{$instance['theme']}.theme.php");

		//timing functions
		$end = microtime(true);
		$time_taken = round(($end-$start)*1000,0);
		echo "\n<!--last fetched: {$instance['generated']} -->\n";
		echo "<!--Generated in $time_taken milliseconds-->\n";
		echo '<!--fidgetr '.FIDGETR_VERSION.'-->'."\n";
	}

	function renderJsonOutput($urlArr) {
		?><script type="text/javascript">
			var fidgetrData_<?php echo $this->number; ?> =
				{
					"photo":
		<?php
			echo $this->array2json($urlArr);
		?>}
		var fidgetrObject_<?php echo $this->number; ?> = eval(fidgetrData_<?php echo $this->number; ?>);</script><?php
	}

	/*array2json provided by bin-co.com under BSD license and modified fairly heavily to more closely imitate json_encode for
	fidgetr's purposes*/
	function array2json($arr) {
		if(function_exists('json_encode')) return json_encode($arr); //Latest versions of PHP already have this functionality.
		$parts = array();
		$is_list = false;

		//Find out if the given array is a numerical array 
		$keys = array_keys($arr);
		$max_length = count($arr)-1;
		if(($keys[0] == 0) and ($keys[$max_length] == $max_length)) {//See if the first key is 0 and last key is length - 1
			$is_list = true;
			for($i=0; $i<count($keys); $i++) { //See if each key correspondes to its position
				if($i != $keys[$i]) { //A key fails at position check.
					$is_list = false; //It is an associative array.
					break;
				}
			}
		}

		foreach($arr as $key=>$value) {
			if(is_array($value)) { //Custom handling for arrays
				if($is_list) $parts[] = $this->array2json($value); /* :RECURSION: */
				else $parts[] = '"' . $key . '":' . $this->array2json($value); /* :RECURSION: */
			} else {
				$str = '';
				if(!$is_list) $str = '"' . $key . '":';

				//Custom handling for multiple data types
				if($value === false) {
					$str .= 'false'; //The booleans
				} elseif($value === true) {
					$str .= 'true';
				//following line heavily modified to more closely imitate native json_encode.  numerics will also be encoded here
				} else {
					$search = array("\'","/","\n","\r");
					$replace = array('\'',"\\/","\\n","\\r");
					$str .= '"' . str_replace($search,$replace,addslashes($value)) . '"'; //All other things 
				// :TODO: Is there any more datatype we should be in the lookout for? (Object?) 
				}
				$parts[] = $str;
			}
		}
		$json = implode(',',$parts);

		if($is_list) return '[' . $json . ']'; //Return numerical JSON
		return '{' . $json . '}'; //Return associative JSON
	}

	//a convenience function to supplement the parent class save_settings(), which saves all instances
	function save_instance_settings($instance) {
		$all_instances = $this->get_settings();
		$all_instances[$this->number] = $instance;
		$this->save_settings($all_instances);
	}

	//a convenience function to supplement the parent class get_settings(), which gets all instances
	function get_instance_settings() {
		$all_instances = $this->get_settings();
		return $all_instances[$this->number];
	}

	//from wp_widget
	function update($new_instance, $old_instance) {
		//if wp super cache is installed we need to invalidate the cache when options are changed.  otherwise let it do its thing.
		if(function_exists('wp_cache_no_postid')) {
			wp_cache_no_postid(0);
		}

		$instance = $old_instance;
		//set title
		$instance['title'] = strip_tags(stripslashes($new_instance['title']));
		//test username/email, only valid flickr names allowed
		if($old_instance['username'] != $new_instance['username']) {
			//username/email has changed so we must test it.
			$f = new flickrApi();
			if($f->getNsid($new_instance['username'])) {
				$instance['username'] = $new_instance['username'];
			}
		}
		//test range # of photos
		if(101 > ((int)$new_instance['num']) && ((int)$new_instance['num']) > 0) {
			$instance['num'] = $new_instance['num'];
		}

		//test cachetime, minimum 10 minutes
		if((int)$new_instance['cachetime'] >= 600) {
			$instance['cachetime'] = $new_instance['cachetime'];
		}

		//load photoset choices...this probably needs caching
		$f = new flickrApi();
		$f->getNsid($instance['username']);
		$f->getPhotosets();
		$photosets[0] = 'Latest Photos';
		if(is_array($f->photosetArr)) {
			foreach($f->photosetArr as $value) {
				$photosets[$value['id']] = $value['title']['_content'];
			}
		}
		//set photoset.
		if(array_key_exists($new_instance['photoset'],$photosets)) {
			$instance['photoset'] = $new_instance['photoset'];
		}

		//set comments flag, default is off so only need to check if it's set now
		if($new_instance['commentsflag']) {
			$instance['commentsflag'] = 1;
		} else {
			$instance['commentsflag'] = '';
		}
		//set theme
		if (in_array($new_instance['theme'],$this->theme_choices)) {
			$instance['theme'] = $new_instance['theme'];
		}

		//default is to 1, so if this is not set we want to unset the pref
		if($new_instance['showtitle']) {
			$instance['showtitle'] = 1;
		} else {
			$instance['showtitle'] = '';
		}

		if($new_instance['slimbox']) {
			$instance['slimbox'] = 1;
		} else {
			$instance['slimbox'] = '';
		}

		if($new_instance['randomize']) {
			$instance['randomize'] = 1;
		} else {
			$instance['randomize'] = '';
		}

		return $instance;
	}

	//from wp_widget
	function form($instance) {
		$default_array = array( 'title' => 'Fidgetr',
								'username' => 'Paul Kehrer',
								'num' => 4,
								'cachetime' => 7200,
								'photoset' => 0,
								'commentsflag' => '',
								'theme' => 'default',
								'showtitle' => 1,
								'slimbox' => 0,
								'randomize' => 0,
								'datacache' => null,
								'generated' => ''
						);
		$instance = wp_parse_args( (array) $instance, $default_array );
		//should really cache photosets some day
		$f = new flickrApi();
		$f->getNsid($instance['username']);
		$f->getPhotosets();
		$photosets[0] = 'Latest Photos';
		if(is_array($f->photosetArr)) {
			foreach($f->photosetArr as $value) {
				$photosets[$value['id']] = $value['title']['_content'];
			}
		}

		?>
		<span class="fidgetr_return">&nbsp;</span>
		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('title'); ?>">
				<?php _e('Widget Title', 'fidgetr'); ?>: 
				<input id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $instance['title']; ?>" class="text widefat" />
			</label>
		</p>

		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('username'); ?>">
				<?php _e('Flickr Username/Email Address/NSID', 'fidgetr'); ?>: 
				<input id="<?php echo $this->get_field_id('username'); ?>" name="<?php echo $this->get_field_name('username'); ?>" type="text" value="<?php echo $instance['username']; ?>" class="text widefat fidgetr_username" />
			</label>
			<span class="fidgetr_loading" style="display:none">
				<img style="vertical-align:middle" src="images/wpspin_light.gif" alt="wspin_light" />
			</span>
		</p>

		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('num'); ?>">
				<?php _e('Number of Photos to Display', 'fidgetr'); ?>: 
				<input id="<?php echo $this->get_field_id('num'); ?>" name="<?php echo $this->get_field_name('num'); ?>" type="text" value="<?php echo $instance['num']; ?>" class="text widefat" />
			</label>
		</p>

		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('cachetime'); ?>">
				<?php _e('Cache Time (in seconds)', 'fidgetr'); ?>: 
				<input id="<?php echo $this->get_field_id('cachetime'); ?>" name="<?php echo $this->get_field_name('cachetime'); ?>" type="text" value="<?php echo $instance['cachetime']; ?>" class="text widefat" />
			</label>
		</p>

		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('photoset'); ?>">
				<?php _e('Photoset', 'fidgetr'); ?>: 
				<select id="<?php echo $this->get_field_id('photoset'); ?>" name="<?php echo $this->get_field_name('photoset'); ?>" class="fidgetr_photoset">
				<?php foreach($photosets as $key=>$value) {
						if($key == $instance['photoset']) {
							$selected = 'selected="selected"';
						}
						echo "<option value=\"$key\" $selected>$value</option>\n";
						unset($selected);
					}
					?>
				</select>
			</label>
		</p>
		<?php
		if($instance['commentsflag']) {
			$comment_checked = 'checked="checked"';
		}
		if($instance['showtitle']) {
			$title_checked = 'checked="checked"';
		}
		if($instance['slimbox']) {
			$slimbox_checked = 'checked="checked"';
		}
		if($instance['randomize']) {
			$randomize_checked = 'checked="checked"';
		}
		?>
		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('commentsflag'); ?>">
				<input id="<?php echo $this->get_field_id('commentsflag'); ?>" name="<?php echo $this->get_field_name('commentsflag'); ?>" type="checkbox" value="1" class="checkbox" <?php echo $comment_checked; ?>/> <?php _e('Fetch Comments', 'fidgetr'); ?>
			</label>
		</p>

		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('showtitle'); ?>">
				<input id="<?php echo $this->get_field_id('showtitle'); ?>" name="<?php echo $this->get_field_name('showtitle'); ?>" type="checkbox" value="1" class="checkbox" <?php echo $title_checked; ?>/> <?php _e('Show Photo Title', 'fidgetr'); ?>
			</label>
		</p>

		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('randomize'); ?>">
				<input id="<?php echo $this->get_field_id('randomize'); ?>" name="<?php echo $this->get_field_name('randomize'); ?>" type="checkbox" value="1" class="checkbox" <?php echo $randomize_checked; ?>/> <?php _e('Randomize Photos', 'fidgetr'); ?>
			</label>
		</p>


		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('slimbox'); ?>">
				<input id="<?php echo $this->get_field_id('slimbox'); ?>" name="<?php echo $this->get_field_name('slimbox'); ?>" type="checkbox" value="1" class="checkbox" <?php echo $slimbox_checked; ?>/> <?php _e('Use Slimbox', 'fidgetr'); ?>
			</label>
		</p>

		<p class="fidgetr_field">
			<label for="<?php echo $this->get_field_name('theme'); ?>">
				<?php _e('Theme', 'fidgetr'); ?>: 
				<select id="<?php echo $this->get_field_id('theme'); ?>" name="<?php echo $this->get_field_name('theme'); ?>" class="fidgetr_theme">
				<?php foreach($this->theme_choices as $value) {
					if($value == $instance['theme']) {
						$selected = 'selected="selected"';
					}
					echo "<option value=\"$value\" $selected>$value</option>\n";
					unset($selected);
				}
				?>
				</select>
			</label>
		</p>
		<?php
	}
}
/* turns out you can't just run register_widget because the widget factory might not be instantiated yet
so let's make a function and add it to the widget_init action in the wordpress event loop */
function fidgetr_register() {
	register_widget('fidgetr_widget');
}

add_action('widgets_init','fidgetr_register');

//pretty safe to assume we would like jquery available for use within all themes
wp_enqueue_script('jquery');
?>