<?php
class flickrApi {
	private $api_key = '37718025c9d25aaa2f1199d8886d43a4';
	private $format = 'php_serial';
	private $user_id = '';

	public $username = '';
	public $urlArr = '';
	public $commentsArr = '';
	public $photosetArr = '';
	public $randomize = 0; //should we randomize (fetch 100, return $num randomly)
	public $num = 10; //number of photos we want to return in the end.
	public $is_pool = false;

	public function getNsid($username) {
		$this->username = $username;
		$pattern = '/^([a-z0-9])(([-a-z0-9._])*([a-z0-9]))*\@([a-z0-9])' .
		'(([a-z0-9-])*([a-z0-9]))+' . '(\.([a-z0-9])([-a-z0-9_-])?([a-z0-9])+)+$/i';
		if(preg_match($pattern,$this->username)) { //regex to match on email address.
			$params['method'] = 'flickr.people.findByEmail';
			$params['find_email'] = $this->username;
		//regex to match an nsid.
		} else if (preg_match('/^[0-9]{7,11}@N[0-9]{2}/',trim($this->username))) { 
			$this->user_id = trim($this->username);
			return true;
		} else {
			$params['method'] = 'flickr.people.findByUsername';
			$params['username'] = $this->username;
		}
		$params['api_key'] = $this->api_key;
		$params['format'] = $this->format;
		$responseObj = $this->apiReq($params);
		if($responseObj) {
			$this->user_id = $responseObj['user']['nsid'];
			return true;
		} else {
			return false;
		}
	}

	public function getPublicPhotos() {
		//instead of using this method we could fetch a feed of this structure, but we can't use it for randomize since we only receive 20 items back max.
		//http://api.flickr.com/services/feeds/photos_public.gne?id=86366039@N00&lang=en-us&format=php_serial
		$per_page = $this->numberOfPhotos();
		$params['method'] = 'flickr.people.getPublicPhotos';
		$params['api_key'] = $this->api_key;
		$params['format'] = $this->format;
		$params['user_id'] = $this->user_id;
		$params['per_page'] = $per_page;
		$params['extras'] = 'url_m';
		$responseObj = $this->apiReq($params);
		if(!$responseObj) {
			//maybe it's actually a photo pool. this could be improved if there's a way to determine if an nsid is a pool without a query
			$responseObj = $this->getPhotosFromPool();
		}
		$this->buildPhotoUrlArray($responseObj['photos']['photo']);
	}
	
	public function getPhotosFromPool() {
		$per_page = $this->numberOfPhotos();
		$this->is_pool = true;
		$params['method'] = 'flickr.groups.pools.getPhotos';
		$params['api_key'] = $this->api_key;
		$params['format'] = $this->format;
		$params['per_page'] = $per_page;
		$params['extras'] = 'url_m';
		$params['group_id'] = $this->user_id;
		return $this->apiReq($params);
	}
	
	//takes a photourlarray and randomizes the order, then truncates
	public function randomizePhotos() {
		shuffle($this->urlArr);
		array_splice($this->urlArr,$this->num,count($this->urlArr));
	}
	
	public function numberOfPhotos() {
		if($this->randomize) {
			//we can up this to a max of 500 to randomize inside that block
			$per_page = 250;
		} else {
			$per_page = $this->num;
		}
		return $per_page;
	}

	public function getPhotosets() {
		$params['method'] = 'flickr.photosets.getList';
		$params['api_key'] = $this->api_key;
		$params['format'] = $this->format;
		$params['user_id'] = $this->user_id;
		$responseObj = $this->apiReq($params);
		$this->photosetArr = $responseObj['photosets']['photoset'];
	}

	public function getPhotosFromPhotoset($photoset_id) {
		$per_page = $this->numberOfPhotos();
		$params['method'] = 'flickr.photosets.getPhotos';
		$params['api_key'] = $this->api_key;
		$params['format'] = $this->format;
		$params['photoset_id'] = $photoset_id;
		$params['per_page'] = $per_page;
		$responseObj = $this->apiReq($params);
		$this->buildPhotoUrlArray($responseObj['photoset']['photo']);
	}

	//this function expects a multidimensional array sourced from getPublicPhotos/getPhotos from photoset
	public function getPhotoComments() {
		$params['method'] = 'flickr.photos.comments.getList';
		$params['api_key'] = $this->api_key;
		$params['format'] = $this->format;
		foreach ($this->urlArr as $key=>$value) {
			$params['photo_id'] = $value['photo_id'];
			$responseObj = $this->apiReq($params);
			if(is_array($responseObj['comments']['comment'])) {
				foreach($responseObj['comments']['comment'] as $commentkey=>$commentdata) {
					$responseObj['comments']['comment'][$commentkey]['_content'] = $commentdata['_content'];
				}
			}
			$this->commentsArr[$value['photo_id']] = $responseObj['comments']['comment']; //add the comments to the comments only array
			$this->urlArr[$key]['comment'] = $responseObj['comments']['comment']; //add the comments to the combined array
		}
	}

	private function apiReq($params) {
		foreach ($params as $key=>$value) {
			$encodedParams[] = urlencode($key).'='.urlencode($value);
		}
		$url = 'http://api.flickr.com/services/rest/?'.implode('&', $encodedParams);
		require_once( ABSPATH . 'wp-includes/class-snoopy.php');
		$snoopy = new Snoopy();
		$result = $snoopy->fetch($url);
		if($result) {
			$response = $snoopy->results;
		}
		$responseObj = unserialize($response);
		if($responseObj['stat'] == "ok") {
			return $responseObj;
		} else {
//			$this->error($responseObj);
			return false;
		}
	}

	private function buildPhotoUrlArray($photo) {
		if(is_array($photo)) {
			foreach ($photo as $value) {
				extract($value);
				/*
				original is not included because frequently this will be disallowed for public 
				users and we don't want to require authentication.
				m_dims currently not being used (info gained via extras=url_m)
				*/
				//this SHOULD mean it is a photo pool since the user_id and owner don't match.
				if($this->is_pool) {
					$flickrUrl = "http://www.flickr.com/photos/$owner/$id/in/pool-{$this->user_id}/";
				} else {
					$flickrUrl = "http://www.flickr.com/photos/{$this->user_id}/$id";
				}
				$urlArr[] = array(
					'url_s'=>"http://farm$farm.static.flickr.com/$server/{$id}_{$secret}_s.jpg",
					'url_t'=>"http://farm$farm.static.flickr.com/$server/{$id}_{$secret}_t.jpg",
					'url_m'=>"http://farm$farm.static.flickr.com/$server/{$id}_{$secret}_m.jpg",
					'url_med'=>"http://farm$farm.static.flickr.com/$server/{$id}_{$secret}.jpg",
					'url_b'=>"http://farm$farm.static.flickr.com/$server/{$id}_{$secret}_b.jpg",
					'm_dims'=>array('width'=>$width_m,'height'=>$height_m),
					'flickr'=>$flickrUrl,
					'title'=>$title,
					'photo_id'=>$id
				);
			}
			$this->urlArr = $urlArr;
			if($this->randomize) {
				$this->randomizePhotos();
			}
		}
	}

	/*
	not currently in use
	*/
	private function error($responseObj) {
		echo 'status: '.$responseObj['stat']."\ncode: ".$responseObj['code']."\nmessage: ".$responseObj['message']."\n";
		exit;
	}
}
?>